from PIL import Image, ImageDraw, ImageFont
import time, re

class ImageEditorServiceClass():
    def ImageDrawText(self, text, sourceImg, targetImg, fontSize=28, fontPath=None, color='#000000', textPosTuple=None, textPosAnchor=None, textPosAnchorOff=None, sizeTuple=None):
        '''
        textPosAnchor + textPosAnchorOff适用于添加封面文字, textPosTuple适用于普通文字
        textPos 坐标在左上角开始,不填默认最中间
        textPosAnchor 坐标锚点，默认最中间 0/默认 中，1 左，2 上，3 右， 4 下
        textPosAnchorOff 偏移
        '''
        # 设置字体样式
        font = ImageFont.truetype(font=fontPath or 'C:/Windows/Fonts/simsun.ttc', size=fontSize)

        # 打开图片
        image = Image.open(sourceImg)
        draw = ImageDraw.Draw(image)
        width, height = image.size

        pos = (0,0)
        if textPosTuple:
            pos = textPosTuple
        else:
            if not textPosAnchor or textPosAnchor == 0:
                pos = ((width-fontSize*len(text))/2, (height-fontSize)/2)
            elif not textPosAnchor or textPosAnchor == 1:
                pos = (0, (height-fontSize)/2)
            elif not textPosAnchor or textPosAnchor == 2:
                pos = ((width-fontSize*len(text))/2, 0)
            elif not textPosAnchor or textPosAnchor == 3:
                pos = (width-fontSize*len(text), (height-fontSize)/2)
            elif not textPosAnchor or textPosAnchor == 4:
                pos = ((width-fontSize*len(text))/2, height-fontSize)
            if textPosAnchorOff:
                pos = (pos[0]+textPosAnchorOff[0], pos[1]+textPosAnchorOff[1])

        draw.text(pos, '%s' % text, color, font)
        # 生成图片
        # image.save(targetImg, 'png')
        image.save(targetImg)

        # 压缩图片
        if sizeTuple:
            sImg = Image.open(targetImg)
            # resize 是裁剪， thumbnail 不会裁剪，是等比缩放
            if sImg.size[0] < sizeTuple[0] and sImg.size[1] < sizeTuple[1]:
                dImg = sImg.resize(sizeTuple, Image.ANTIALIAS)
                dImg.save(targetImg)
            else:
                sImg.thumbnail(sizeTuple, Image.ANTIALIAS)
                sImg.save(targetImg)

    #切图
    def ImageCut(self, imgPath, saveDir, saveName, hCutNum, vCutNum=1):
        image = Image.open(imgPath)
        width, height = image.size
        item_width = int(width / hCutNum)
        item_height = int(height / vCutNum)
        box_list = []    
        # (left, upper, right, lower)
        if vCutNum and vCutNum != 1 :
            for j in range(vCutNum):
                for i in range(hCutNum):
                    box = (item_width * i, item_height * j, item_width * (i + 1), item_height * (j + 1))
                    box_list.append(box)
        else:
            for i in range(hCutNum):
                box = (i*item_width,0,(i+1)*item_width,height)
                box_list.append(box)

                
        image_list = [image.crop(box) for box in box_list]    
        #保存
        index = 1
        for image in image_list:
            image.save('%s%s'%(saveDir,fileName)+str(index) + suffixName)
            index += 1

if __name__ == "__main__":
    import os
    ImageEditorService = ImageEditorServiceClass()
    # ImageEditorService.ImageDrawText('哈哈哈哈哈哈哈哈哈哈', './book.png', 'TestRes/1+Text.png', textPosAnchor=0)
    # os.system('start TestRes/1+Text.png')

    tartgetImg = r'book/cover/'
    dirPath = r'book/source/bg/'
    fileName = 'book'
    suffixName = '.png'
    desc = '当一个人真正投入爱情，那么她也就一定会变成笨蛋。因为爱情的悸动和占有欲，会焚烧人内心所有理智，令人变成蠢蛋。而这恰恰是恋爱最好的地方。若是你发现爱人在恋爱时一直都是表现的很理智很聪明，就要小心了。'
    spliteNum = 14
    maxNum = 98
    def cut_text(text,lenth):
        textArr = re.findall('.{'+str(lenth)+'}', text)
        textArr.append(text[(len(textArr)*lenth):])
        return textArr
    ImageEditorService.ImageDrawText('《%s》'%fileName, '%s%s%s'%(dirPath, fileName, suffixName), '%s%s%s'%(tartgetImg, fileName, suffixName), textPosAnchor=0, fontSize=20, color='#000000', sizeTuple=(1728,1024))
    
    ImageEditorService.ImageCut('%s%s%s'%(tartgetImg, fileName, suffixName), tartgetImg, fileName, 3, 2)
    # ImageEditorService.ImageCut('%s%s%s'%(tartgetImg, fileName, suffixName), tartgetImg, fileName, 3)



from os import remove
from HttpService import HttpServiceClass
from FileService import FileServiceClass
import json

class ToolsServiceClass(object):
    def __init__(self):
        self.httpService = HttpServiceClass()
        self.fileService = FileServiceClass()
        #region 企业微信
        self.corpid = "wwb6ec4438ccc388c3"      # Corpid是企业号的标识
        self.secret = "_surPYo0SLVcxPKzAnHdWBwpGl2woxWKWpSTMvbZvnw"     # Secret是管理组凭证密钥
        self.agentid = "1000002"        # 应用ID
        self.tokenConfig = r'./temp/zabbix_wechat_config.json'     # token_config文件放置路径
        self.fileService.CreateDir(self.tokenConfig)
        self.bot = "https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key=c2ae20ef-9a26-4791-abb5-962e07f97144"
        #endregion
    def ToNumberCompany(self, number):
        if number > 100000000:
            return number/100000000 + '亿'
        elif number > 10000000:
            return number/10000000 + '千万'
        elif number > 1000000:
            return number/1000000 + '百万'
        elif number > 100000:
            return number/100000 + '十万'
        elif number > 10000:
            return number/10000 + '万'
        else:
            return number
    
    #https://www.runoob.com/markdown/md-block.html
    def SendNotify(self, desc=''):
        self.SendWorkWX({"msgtype" : "text", "text" : {"content" : desc}})
    
    def _GetTokenFromServer(self, corpid, secret, tokenConfig):
        """ 获取 token """
        token = False
        try:
            file = open(tokenConfig, 'r')
            token = json.load(file)['access_token']
            file.close()
        except:
            url = "https://qyapi.weixin.qq.com/cgi-bin/gettoken"
            data = {
                "corpid": corpid,
                "corpsecret": secret
            }
            r = self.httpService.PostUrlResetByRequests(url=url, params=data, verify=False)
            print("企业微信Token: %s"%r.json())
            if r.json()['errcode'] != 0:
                token = False
            else:
                token = r.json()['access_token']
                file = open(tokenConfig, 'w')
                file.write(r.text)
                file.close()
        return token

    def SendWorkWX(self, data, corpid=None, secret=None, errorNum=3):
        """ 
        发送消息到企业微信
        API文档： https://work.weixin.qq.com/api/doc/90000/90135/90236
        """
        if not corpid or not secret:
            corpid = self.corpid
            secret = self.secret
            tokenConfig = self.tokenConfig
        else:
            tokenConfig = r'./temp/%s_wechat_config.json'%(corpid)
        # 获取token信息
        token = self._GetTokenFromServer(corpid, secret, tokenConfig)
        # 发送消息
        if not "agentid" in data:
            data["agentid"] = self.agentid
        if not "touser" in data:
            data["touser"] = "@all"
        if not "toparty" in data:
            data["toparty"] = "1"
        url = "https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token=%s" % token
        result = self.httpService.PostUrlResetByRequests(url=url, data=json.dumps(data), verify=False)
        print("企业微信应用消息： %s"%result.json())
        # 如果发送失败，将重试多次if
        if result.json()['errcode'] != 0 and errorNum and errorNum > 0:
            if result.json()['errcode'] == 42001:
                remove(tokenConfig)
                token = self._GetTokenFromServer(corpid, secret, tokenConfig)
            return self.SendWorkWX(data, corpid, secret, errorNum-1)
        else:
            return result.json()
    
    def SendWorkWXBot(self, data, url=None):
        """
        发送企业微信消息，机器人群
        API文档： https://work.weixin.qq.com/api/doc/90000/90136/91770
        """
        url = url or self.bot
        if not "mentioned_list" in data["text"]:
            data["text"]["mentioned_list"] = ["@all"]
        if not "mentioned_mobile_list" in data["text"]:
            data["text"]["mentioned_mobile_list"] = ["@all"]
        print(json.dumps(data))
        result = self.httpService.PostUrlResetByRequests(url=url, data=json.dumps(data), verify=False)
        print("企业微信应用消息： %s"%result.json())



from moviepy.editor import *
from natsort import natsorted
from PIL import Image
import os, glob, hashlib, shutil
class VideoEditorServiceClass():
    # 视频截取
    def video_subclip(self, video_path, save_path, start_pos, end_pos, is_relative_end_pos = False):
        tempPath, save_path = self._video_path_handle(video_path, save_path)
        video = VideoFileClip(video_path)
        if is_relative_end_pos:
            # 剪辑视频，从x秒开始到视频结尾前x秒
            video = video.subclip(start_pos, video.duration-end_pos)
        else:
            # 剪辑视频，截取视频前x秒
            video = video.subclip(start_pos, end_pos)
        video.to_videofile(save_path, fps=25, remove_temp=True)
        return self.result_param(True, save_path, video, None, '')

    # 合成文件夹下所有的视频
    def video_merge_all_video_by_dir(self, dir_path, save_path, suffix_name='.mp4'):
        video_list = []
        # 访问 video 文件夹 (假设视频都放在这里面)
        for root, dirs, files in os.walk(dir_path):
            # 按文件名排序
            # files.sort() # 原 1.mp4, 2.mp4, 10.mp4.  排序后  1.mp4, 10.mp4, 2.mp4.
            files = natsorted(files) # 原 1.mp4, 2.mp4, 10.mp4. 排序后  1.mp4, 2.mp4, 10.mp4.
            # 遍历所有文件
            for file in files:
                # 如果后缀名为 .mp4
                if os.path.splitext(file)[1] == suffix_name:
                    # 拼接成完整路径
                    filePath = os.path.join(root, file)
                    video_list.append(filePath)
        if len(video_list) == 0:
            return self.result_param(False, save_path, None, None, '列表为空')
        return self.video_merge_all_video_by_list(video_list, save_path, suffix_name)

    # 合成列表所有的视频
    def video_merge_all_video_by_list(self, video_list, save_path, suffix_name='.mp4'):
        # 定义一个数组
        target_list= []
        # 遍历所有文件
        for file_path in video_list:
            # 如果后缀名为 .mp4
            if os.path.splitext(file_path)[1] == suffix_name:
                # 载入视频
                video = VideoFileClip(file_path)
                # 添加到数组
                target_list.append(video)
        if len(target_list) == 0:
            return self.result_param(False, save_path, None, None, '文件列表是空的')
        try:
            # 拼接视频
            final_clip = concatenate_videoclips(target_list)
            # 生成目标视频文件
            final_clip.to_videofile(save_path, fps=25, remove_temp=True)
            return self.result_param(True, save_path, final_clip, None, '')
        except BaseException as e:
            return self.result_param(False, save_path, None, None, str(e))

    # 视频播放区域裁剪
    def video_crop(self, video_path, save_path, x_center, y_center, width, height):
        video = VideoFileClip(video_path)
        final_clip = video.crop(x_center=x_center, y_center=y_center, width=width, height=height)
        final_clip.to_videofile(save_path, fps=25, remove_temp=True)
        return self.result_param(True, save_path, final_clip, None, '')

    # 改变视频分辨率,可能花屏
    def video_resize(self, video_path, save_path, width, height):
        video = VideoFileClip(video_path)
        final_clip = video.resize(newsize=(width, height))
        final_clip.to_videofile(save_path, fps=25, remove_temp=True)
        return self.result_param(True, save_path, final_clip, None, '')

    # 把文件夹下所有图片合成视频
    def video_images_clip_by_dir(self, dir_path, save_path, fps=25, suffix_name='.jpg'):
        pic_list = []
        for root, dirs, files in os.walk(dir_path):
            # 按文件名排序
            # files.sort() # 原 1.mp4, 2.mp4, 10.mp4.  排序后  1.mp4, 10.mp4, 2.mp4.
            files = natsorted(files) # 原 1.mp4, 2.mp4, 10.mp4. 排序后  1.mp4, 2.mp4, 10.mp4.
            # 遍历所有文件
            for file in files:
                # 如果后缀名为 .mp4
                if os.path.splitext(file)[1] == suffix_name:
                    # 拼接成完整路径
                    filePath = os.path.join(root, file)
                    pic_list.append(filePath)
        if len(pic_list) == 0:
            return self.result_param(False, save_path, None, None, '列表为空')
        return self.video_images_clip_by_list(pic_list, save_path, fps, suffix_name)

    # 把图片列表合成视频
    def video_images_clip_by_list(self, pic_list, save_path, fps=25, suffix_name='.jpg'):
        target_list= []
        for pic_path in pic_list:
            if os.path.splitext(pic_path)[1] == suffix_name:
                # 添加到数组
                target_list.append(pic_path)
        if len(target_list)==0:
            return self.result_param(False, save_path, None, None, '列表为空')
        try:
            final_clip = ImageSequenceClip(pic_list, fps=fps)
            final_clip.to_videofile(save_path, fps=fps, remove_temp=True)
        except BaseException as e:
            print("很大可能是图片不一样大，可以调用images_resize_by_dir 或者 images_resize_by_list 统一设置图片大小")
            return self.result_param(False, save_path, None, None, str(e))
        return self.result_param(True, save_path, final_clip, None, '')

    # 把两个视频放在一个画面上同时播放(左右)
    def video_merge_video_to_one_canvas(self, video_list, save_path):
        video1 = VideoFileClip(video_list[0])
        video2 = VideoFileClip(video_list[1])
        # 注意：.set_position([x1, y1])中的x1，y1为视频左上角的坐标
        target1 = video1.set_position([0, 0])
        target2 = video2.set_position([video1.w, 0])
        final_clip = CompositeVideoClip([target1, target2], size=(video1.w+video2.w, video1.h))
        final_clip.to_videofile(save_path, fps=25, remove_temp=True)
        return self.result_param(True, save_path, final_clip, None, '')

    # 把两个视频放在一个画面上同时播放(自定义位置)
    def video_merge_video_to_one_canvas_by_pos(self, video_list, save_path, position):
        video1 = VideoFileClip(video_list[0])
        video2 = VideoFileClip(video_list[1])
        # 注意：.set_position([x1, y1])中的x1，y1为视频左上角的坐标
        target1 = video1.set_position([0, 0])
        target2 = video2.set_position(position)
        final_clip = CompositeVideoClip([target1, target2], size=(video1.w+video2.w, video1.h))
        final_clip.to_videofile(save_path, fps=25, remove_temp=True)
        return self.result_param(True, save_path, final_clip, None, '')

    # 路径需要带后缀 VideoFileClipPathList
    def video_merge_list_all_video(self, video_path_list, save_path):
        clips = []
        clip = None
        for videoPath in video_path_list:
            try:
                clip = VideoFileClip(videoPath)
            except BaseException as e:
                print("video_merge_list_all_video Error " + str(e))
                clip = None
            finally:
                if clip != None:
                    clips.append(clip)
        if len(clips) == 0 :
            return self.result_param(False, save_path, None, None, '列表为空')
        
        final_clip = concatenate_videoclips(clips)
        # mp4文件默认用libx264编码， 比特率单位bps
        # final_clip.write_videofile(save_path, codec="libx264", bitrate="10000000") 
        final_clip.to_videofile(save_path, fps=25, remove_temp=True)
        return self.result_param(True, save_path, final_clip, None, '')

    # 改变视频MD5，路径需要带后缀 VideoFileClipPath
    def video_change_md5(self, videoPath, save_path):
        tempPath, save_path = self._video_path_handle(videoPath, save_path)
        return self.video_merge_list_all_video([tempPath], save_path)

    # 视频路径处理，重名那种
    def _video_path_handle(self, videoPath, save_path):
        tempPath = videoPath
        if videoPath == save_path:
            tempFileName = "tempVideo"
            [dirName,fileName]=os.path.split(videoPath)
            fileName = "-y.mp4"
            if dirName == "":
                tempPath = "{}_{}".format(tempFileName,fileName)
            else:
                tempPath = "{}/{}_{}".format(dirName,tempFileName,fileName)
            shutil.copyfile(videoPath,tempPath)
        return tempPath, save_path

    # 视频添加LOG  VideoClipAddLogoPath
    def video_add_log(self, videoPath, save_path, logoPngPath = None, logoTextStr = None, color='white', align='center', fontsize = 25, left=0, top=0, right=0, bottom=0, opacity=0, pos=('center','center'), fontPath=None):
        tempPath, save_path = self._video_path_handle(videoPath, save_path)
        if logoPngPath == None and logoTextStr == None:
            return self.result_param(False, save_path, None, None, '没有设置水印')

        clipList = []        
        video = VideoFileClip(tempPath)
        clipList.append(video)
        logoPngClip = None
        logoTextClip = None
        if logoPngPath != None:
            logoPngClip = (ImageClip(logoPngPath)
                    .set_duration(video.duration)  # 水印持续时间
                    .resize(height=50)  # 水印的高度，会等比缩放
                    .margin(left=left, top=top, right=right, bottom=bottom, opacity = opacity)# 水印边距和透明度
                    .set_pos(pos))  # 水印的位置
            clipList.append(logoPngClip)

        if logoTextStr != None:
            FONT_URL = fontPath or r'C:/Windows/Fonts/simsun.ttc'  #r'C:\Windows\Fonts\simhei.ttf'    #C:\Windows\Fonts\STXINGKA.TTF'
            if not os.path.exists(FONT_URL):
                print("字体文件不存在 %s"%FONT_URL)
                return False
            logoTextClip = (TextClip(txt=logoTextStr, color=color, size=(640, 480), method='caption',
                    align=align, fontsize=fontsize, font=FONT_URL)
                    .set_duration(video.duration)
                    .margin(left=left, top=top, right=right, bottom=bottom, opacity = opacity)
                    .set_pos(pos))  # 水印的位置
            clipList.append(logoTextClip)

        if len(clipList) == 0:
            return self.result_param(False, save_path, None, None, '列表为空')
        final_clip = CompositeVideoClip(clipList)
        # mp4文件默认用libx264编码， 比特率单位bps
        # final_clip.write_videofile(save_path, codec="libx264", bitrate="10000000") 
        final_clip.to_videofile(save_path, fps=25, remove_temp=True)
        return self.result_param(True, save_path, final_clip, None, '')

    # 视频添加封面
    def video_add_cover(self, pic_list, video_path, save_path, cover_video_path=None, fps=25, suffix_name='.jpg'):
        video = VideoFileClip(video_path)
        if not cover_video_path:
            cover_video_path = save_path
        for item in pic_list:
            img = Image.open(item)
            if video.size != list(img.size):
                try:
                    new_img = img.resize((video.w, video.h), Image.BILINEAR)
                    new_img.save(item)
                except Exception as e:
                    print(e)
            
        result = self.video_images_clip_by_list(pic_list, cover_video_path, fps, suffix_name)
        if result and result['status']:
            return self.video_merge_all_video_by_list([cover_video_path, video_path], save_path)
        return self.result_param(False, save_path, None, result, result['msg'])

    # 视频添加封面
    def video_add_cover_by_num(self, pic, video_path, save_path, cover_video_path=None, picNum=25, fps=25, suffix_name='.jpg'):
        picList = []
        for item in range(0, picNum):
            picList.append(pic)
        else:
            picList.append(pic)
        return self.video_add_cover(picList, video_path, save_path, cover_video_path, fps, suffix_name)

    # 提取视频中的音频
    def video_get_audio(self, video_path, out_audio_path):
        video = VideoFileClip(video_path)
        audio = video.audio
        audio.write_audiofile(out_audio_path)
        return self.result_param(True, out_audio_path, video, audio, '')

    # 为视频添加另一个视频的音频  source 声音添加到 target
    def video_add_audio_by_video(self, video_path_source, video_path_target, out_video_path):
        video_source = VideoFileClip(video_path_source)
        video_target = VideoFileClip(video_path_target)
        audio = video_source.audio
        videoclip = video_target.set_audio(audio)
        videoclip.write_videofile(out_video_path, audio_codec="aac")
        return self.result_param(True, out_video_path, videoclip, None, '')

    # 为视频添加的音频
    def video_add_audio_by_audio(self, video_path, audio_path, out_video_path):
        video = VideoFileClip(video_path)
        audio = AudioFileClip(audio_path).subclip(0, video.duration)
        videoclip = video.set_audio(audio)
        videoclip.write_videofile(out_video_path, audio_codec="aac")
        return self.result_param(True, out_video_path, videoclip, None, '')

    # --------------------------------------------Tools Start------------------------------------------------
    # 常规返回数据类型
    def result_param(self, status, path, video, obj, msg):
        return {'status':status, 'path': path, 'video': video, 'obj': obj, 'msg': msg}

    # 获取某个文件夹下的所有指定后缀的文件
    def get_dir_all_suffix_name_file(self, dir_path, suffix_name='.png'):
        target_list = []
        for jpgfile in glob.glob(dir_path+'*'+suffix_name):
            target_list.append(jpgfile)
        return target_list

    # 改变某个文件夹下所有指定后缀文件的大小，并保存到指定文件夹下
    def images_resize_by_dir(self, dir_path, save_dir_path, width, height, suffix_name='.png'):
        source_list = self.get_dir_all_suffix_name_file(dir_path, suffix_name)
        for img_path in source_list:
            img_dir = os.path.dirname(img_path)
            if not os.path.exists(img_dir):
                os.makedirs(img_dir)
            img = Image.open(img_path)
            try:
                new_img = img.resize((width, height), Image.BILINEAR)
                new_img.save(os.path.join(save_dir_path, os.path.basename(img_path)))
            except Exception as e:
                print(e)
        return self.result_param(True, save_dir_path, None, None, '')

    #大文件的MD5值
    def video_file_md5(self, filename):
        if not os.path.isfile(filename):
            return
        myhash = hashlib.md5()
        f = open(filename,'rb')
        while True:
            b = f.read(8096)
            if not b :
                break
            myhash.update(b)
        f.close()
        return myhash.hexdigest()
# 日志系统
# __log = LogServiceClass(tag="VideoEditorServiceClass")
# LogD = __log.LogD
# LogW = __log.LogW
# LogE = __log.LogE
# --------------------------------------------Tools End-------------------------------------------------------

if __name__ == "__main__":
    dir_path = os.path.dirname(__file__)
    VideoEditorService = VideoEditorServiceClass()
    # print(VideoEditorService.video_subclip('%s/video/6718334266993855751-y.mp4'%dir_path, '%s/testinput/video_subclip.mp4'%dir_path, 0, 2, True))
    # print(VideoEditorService.video_subclip('%s/video/6763209833844968715.mp4'%dir_path, '%s/testinput/video_subclip.mp4'%dir_path, 4, 2, True))
    
    # print(VideoEditorService.video_merge_all_video_by_dir('%s/video/'%dir_path, '%s/testinput/video_merge_all_video_by_dir.mp4'%dir_path))
    # print(VideoEditorService.video_crop('%s/video/6763209833844968715.mp4'%dir_path, '%s/testinput/video_crop.mp4'%dir_path, 0, 0, 100, 100)) # 未成功
    # print(VideoEditorService.video_resize('%s/video/6763209833844968715.mp4'%dir_path, '%s/testinput/video_resize.mp4'%dir_path, 300, 300))

    # VideoEditorService.images_resize_by_dir('%s/restricted/'%dir_path, '%s/resizeimg/'%dir_path, 100, 100, '.png')
    # print(VideoEditorService.video_images_clip_by_dir('%s/resizeimg'%dir_path, '%s/testinput/video_images_clip_by_dir.mp4'%dir_path, 10, '.png'))

    # print(VideoEditorService.video_merge_video_to_one_canvas(['%s/video/6763209833844968715.mp4'%dir_path, '%s/video/6763449160764034315.mp4'%dir_path], '%s/testinput/video_merge_video_to_one_canvas.mp4'%dir_path))


from datetime import datetime,date
from apscheduler.schedulers.blocking import BlockingScheduler
from apscheduler.schedulers.background import BackgroundScheduler

#https://www.jianshu.com/p/4f5305e220f0

class APSchedulerServiceClass():
    def __init__(self, showLog=False):
        self.blockingScheduler = BlockingScheduler()
        self.backgroundScheduler = BackgroundScheduler()
        self.blockingRecodeDic = {}
        self.backgroundRecodeDic = {}
        self.blockingRecodeIndex = 0
        self.backgroundRecodeIndex = 0
        self.showLog = showLog

    def AddJob_FixedTime(self, job_function, run_date, args=None, kwargs=None, timezone=None, isBackground=True, jobId=None):
        '''
        添加调度器【固定时间任务】

        参数                                说明
        job_function                        事件
        run_date (datetime 或 str)          作业的运行日期或时间 date(2009, 11, 6)，datetime(2009, 11, 6, 16, 30, 5)，'2009-11-06 16:30:05'
        args                                参数
        timezone (datetime.tzinfo 或str)    时区
        jitter (int)                        振动弧度，执行时给个随机时间值，防止峰值
        '''
        jobId, sched = self._CheckGetSchedAndId(isBackground, jobId)
        if not jobId or not sched:
            return False
        job = sched.add_job(job_function, 'date', args=args, kwargs=kwargs, run_date=run_date, timezone=timezone, id=jobId)
        return self.OnAddSchedInfo(jobId, job)

    def AddJob_Interval(self, job_function, args=None, kwargs=None, weeks=0, days=0, hours=0, minutes=0, seconds=0, start_date=None, end_date=None, timezone=None, isBackground=True, jobId=None):
        '''
        添加调度器【周期触发任务】

        参数                                说明
        job_function                        事件
        args                                参数
        weeks (int)                         间隔几周
        days (int)                          间隔几天
        hours (int)                         间隔几小时
        minutes (int)                       间隔几分钟
        seconds (int)                       间隔多少秒
        start_date (datetime 或 str)        开始日期
        end_date (datetime 或 str)          结束日期
        timezone (datetime.tzinfo 或str)    时区
        jitter (int)                        振动弧度，执行时给个随机时间值，防止峰值
        '''
        jobId, sched = self._CheckGetSchedAndId(isBackground, jobId)
        if not jobId or not sched:
            return False
        job = sched.add_job(job_function, 'interval', args=args, kwargs=kwargs, weeks=weeks, days=days, hours=hours, minutes=minutes, seconds=seconds, start_date=start_date, end_date=end_date, timezone=timezone, id=jobId)
        return self.OnAddSchedInfo(jobId, job)

    def AddJob_Cron(self, job_function, args=None, kwargs=None, year=0, month=0, day=0, week=0, day_of_week=0, hour=0, minute=0, second=0, start_date=None, end_date=None, timezone=None, isBackground=True, jobId=None):
        '''
        添加调度器【特定周期触发任务】

        参数                                说明
        job_function                        事件
        args                                参数
        year (int 或 str)                   年，4位数字
        month (int 或 str)                  月 (范围1-12)
        day (int 或 str)                    日 (范围1-31
        week (int 或 str)                   周 (范围1-53)
        day_of_week (int 或 str)            周内第几天或者星期几 (范围0-6 或者 mon,tue,wed,thu,fri,sat,sun)
        hour (int 或 str)                   时 (范围0-23)
        minute (int 或 str)                 分 (范围0-59)
        second (int 或 str)                 秒 (范围0-59)
        start_date (datetime 或 str)        最早开始日期(包含)
        end_date (datetime 或 str)          最晚结束时间(包含)
        timezone (datetime.tzinfo 或str)    指定时区
        jitter (int)                        振动弧度，执行时给个随机时间值，防止峰值

        上面str表达式类型
        表达式     参数类型        描述
        *           所有        通配符。例：minutes=*即每分钟触发
        */a         所有        可被a整除的通配符。
        a-b         所有        范围a-b触发
        a-b/c       所有        范围a-b，且可被c整除时触发
        xth y       日          第几个星期几触发。x为第几个，y为星期几
        last x      日          一个月中，最后个星期几触发
        last        日          一个月最后一天触发
        x,y,z       所有        组合表达式，可以组合确定值或上方的表达式

        '''
        jobId, sched = self._CheckGetSchedAndId(isBackground, jobId)
        if not jobId or not sched:
            return False
        job = sched.add_job(job_function, 'cron', args=args, kwargs=kwargs, year=year, month=month, day=day, week=week, day_of_week=day_of_week, hour=hour, minute=minute, second=second, start_date=start_date, end_date=end_date, timezone=timezone, id=jobId)
        return self.OnAddSchedInfo(jobId, job)

    def RemoveJob(self, jobId='', isAll=False, isBackground=True):
        if isBackground:
            if isAll:
                for item in self.backgroundRecodeDic:
                    if self.backgroundScheduler.get_job(item):
                        self.backgroundScheduler.remove_job(item)
                self.backgroundScheduler.remove_all_jobs()
                self.backgroundRecodeDic.clear()
                self._Log("移除all：%s"%isAll)
            else:
                self.backgroundScheduler.remove_job(jobId)
                self.backgroundRecodeDic.pop(jobId)
                self._Log("移除jobId：%s"%jobId)
            if self.showLog:
                self.backgroundScheduler.print_jobs()
        else:
            if isAll:
                for item in self.blockingRecodeDic:
                    if self.blockingScheduler.get_job(item):
                        self.blockingScheduler.remove_job(item)
                self.blockingScheduler.remove_all_jobs()
                self.blockingRecodeDic.clear()
                self._Log("移除all：%s"%isAll)
            else:
                self.blockingScheduler.remove_job(jobId)
                self.blockingRecodeDic.pop(jobId)
                self._Log("移除jobId：%s"%jobId)
            if self.showLog:
                self.blockingScheduler.print_jobs()
    
    def PauseJob(self, jobId='', isAll=False, isBackground=True):
        '''暂停运行任务'''
        if isBackground:
            if isAll:
                for item in self.backgroundRecodeDic:
                    if self.backgroundScheduler.get_job(item):
                        self.backgroundScheduler.pause_job(item)
                self.backgroundScheduler.pause()
            else:
                self.backgroundScheduler.pause_job(jobId)
        else:
            if isAll:
                for item in self.blockingRecodeDic:
                    if self.backgroundScheduler.get_job(item):
                        self.backgroundScheduler.pause_job(item)
                self.blockingScheduler.pause()
            else:
                self.blockingScheduler.pause_job(jobId)

    def ResumeJob(self, jobId, isAll=False, isBackground=True):
        '''重新/继续运行任务'''
        if isBackground:
            if isAll:
                for item in self.backgroundRecodeDic:
                    if self.backgroundScheduler.get_job(item):
                        self.backgroundScheduler.resume_job(item)
                self.backgroundScheduler.resume()
            else:
                self.backgroundScheduler.resume_job(jobId)
        else:
            if isAll:
                for item in self.blockingRecodeDic:
                    if self.blockingScheduler.get_job(item):
                        self.blockingScheduler.resume_job(item)
                self.blockingScheduler.resume()
            else:
                self.blockingScheduler.resume_job(jobId)

    def Run(self, isRunBackgroundScheduler, isRunBlockingScheduler):
        if isRunBackgroundScheduler:
            if not self.backgroundScheduler.running:
                self.backgroundScheduler.start()
            else:
                self._Log("backgroundScheduler 正在运行")
                if self.showLog:
                    self.backgroundScheduler.print_jobs()
        if isRunBlockingScheduler:
            if not self.blockingScheduler.running:
                self.blockingScheduler.start()
            else:
                self._Log("blockingScheduler 正在运行")
                if self.showLog:
                    self.blockingScheduler.print_jobs()
    
    def _CheckGetSchedAndId(self, isBackground=True, jobId=None):
        '''根据 id 和 状态 判断'''
        sched = False
        if isBackground:
            if jobId:
                if jobId in self.backgroundRecodeDic:
                    print('该已存在于定时器中')
                    return False, False
            else:
                jobId = self.backgroundRecodeIndex + 1
            self.backgroundRecodeIndex += 1
            sched = self.backgroundScheduler
        else:
            if jobId:
                if jobId in self.blockingRecodeDic:
                    print('该已存在于定时器中')
                    return False, False
            else:
                jobId = self.blockingRecodeIndex + 1
            self.blockingRecodeIndex += 1
            sched = self.blockingScheduler
        return str(jobId), sched

    def OnAddSchedInfo(self, jobId, job, isBackground=True):
        '''记录定时器到字典'''
        if isBackground:
            if jobId in self.backgroundRecodeDic:
                print('该已存在于定时器中')
                return False
            self.backgroundRecodeDic[jobId] = job
        else:
            if jobId in self.blockingRecodeDic:
                print('该已存在于定时器中')
                return False
            self.blockingRecodeDic[jobId] = job
        return (jobId, job)

    def _Log(self, value):
        if self.showLog:
            print(value)

if __name__ == "__main__":
    def Test(x):
        print('Log %s'%x)
    apSchedulerService = APSchedulerServiceClass()
    print(apSchedulerService.AddJob_Interval(Test, args=[0],seconds=10))
    print(apSchedulerService.AddJob_Interval(Test, args=[1],seconds=10))
    print(apSchedulerService.AddJob_Interval(Test, args=['33'],seconds=10,jobId='33'))
    apSchedulerService.Run(True, False)
    import time 
    time.sleep(30)
    apSchedulerService.RemoveJob('1')
    apSchedulerService.RemoveJob('2')
    apSchedulerService.RemoveJob('33')
    apSchedulerService.RemoveJob('',True)
    time.sleep(30)


import zlib
import os
import zipfile
# http://api.1473.cn/LearningMaterials/Node/Node_zlib.aspx
# https://www.cnblogs.com/ManyQian/p/9193199.html

class ZipServiceClass():
    # zlib.compress 用来压缩字符串的bytes类型
    def str_compress(self, message):
        '''字符串加密'''
        bytes_message = message.encode()
        return zlib.compress(bytes_message, zlib.Z_BEST_COMPRESSION)
        
    def str_decompress(self, compressed):
        '''字符串解密'''
        return zlib.decompress(compressed)

    # zlib.compressobj 用来压缩数据流，用于文件传输 level[1-9]
    def file_compress(self, inputFile, zlibFile, level=9):
        '''文件加密'''
        infile = open(inputFile, "rb")
        zfile = open(zlibFile, "wb")
        compressobj = zlib.compressobj(level)   # 压缩对象
        data = infile.read(1024)                # 1024为读取的size参数
        while data:
            zfile.write(compressobj.compress(data))     # 写入压缩数据
            data = infile.read(1024)        # 继续读取文件中的下一个size的内容
        zfile.write(compressobj.flush())    # compressobj.flush()包含剩余压缩输出的字节对象，将剩余的字节内容写入到目标文件中
    
    def file_decompress(self, zlibFile, endFile):
        '''文件解密'''
        zlibFile = open(zlibFile, "rb")
        endFile = open(endFile, "wb")
        decompressobj = zlib.decompressobj()
        data = zlibFile.read(1024)
        while data:
            endFile.write(decompressobj.decompress(data))
            data = zlibFile.read(1024)
        endFile.write(decompressobj.flush())
    

    def get_zip_file(self, input_path, result):
        """
        对目录进行深度优先遍历
        :param input_path:
        :param result:
        :return:
        """
        files = os.listdir(input_path)
        for file in files:
            if os.path.isdir(input_path + '/' + file):
                self.get_zip_file(input_path + '/' + file, result)
            else:
                result.append(input_path + '/' + file)

    def zip_file_path(self, input_path, output_path, output_name, password):
        """
        压缩文件夹
        :param input_path: 要被压缩的文件夹路径
        :param output_path: 打出得解压包的路径
        :param output_name: 压缩包名称
        :param password: 密码
        :return:
        """
        zipfiles = zipfile.ZipFile(output_path + '/' + output_name, 'w', zipfile.ZIP_DEFLATED)
        filelists = []
        self.get_zip_file(input_path, filelists)
        for file in filelists:
            zipfiles.write(file)
        zipfiles.setpassword((password or 'zq162082').encode())
        # 调用了close方法才会保证完成压缩
        zipfiles.close()
        return output_path + r"/" + output_name

    def unzip_file_path(self, output_path, input_path, password, output_name=None):
        """
        解压缩文件夹
        :param output_path: 解压缩包的路径
        :param input_path: 压缩文件的路径
        :param password: 密码
        :param output_name: 解压缩包的名字，默认是压缩包的名字
        :return:
        """
        zipfiles=zipfile.ZipFile(input_path,'r')
        zipfiles.extractall(os.path.join(output_path,output_name or os.path.splitext(input_path)[0]),pwd=(password or 'zq162082').encode())
        zipfiles.close()


if __name__=="__main__":
    zipService = ZipServiceClass()
    #-----------------------------str---------------------------------
    print(zipService.str_compress('666666666666669'))
    print(zipService.str_decompress(zipService.str_compress('666666666666669')))
    #-----------------------------strend---------------------------------
    #-----------------------------file---------------------------------
    # 测试数据流压缩
    beginFile = "./beginFile.txt"
    zlibFile = "./zlibFile.txt"
    level = 9
    zipService.file_compress(beginFile, zlibFile, level)
 
    # 测试数据流解压
    zlibFile = "./zlibFile.txt"
    endFile = "./endFile.txt"
    zipService.file_decompress(zlibFile, endFile)
    #-------------------------------fileend------------------------------

    #-----------------------------dir---------------------------------
    zipService.zip_file_path(r"./proxy", os.getcwd(), '123.zip', '1620829248')
    zipService.unzip_file_path(os.getcwd(), '123.zip', '1620829248', '')
    #-------------------------------dirend------------------------------


from HttpService import HttpServiceClass
from TimeService import TimeServiceClass
from FileService import FileServiceClass
from LogService import LogServiceClass
from RegexService import RegexServiceClass
from bs4 import BeautifulSoup
import enum,json,time
# https://www.cnblogs.com/xiaonq/p/8044211.html
# https://www.crummy.com/software/BeautifulSoup/bs4/doc.zh/

class ProxySourceType(enum.Enum):
    XICI = 'XICI',
    IP66 = 'IP66',
    KUAIDAILI = 'KUAIDAILI',

class ProxyServerClass():
    def __init__(self):
        self.http = HttpServiceClass()
        self.time = TimeServiceClass()
        self.file = FileServiceClass()
        self.log = LogServiceClass()
        self.regex = RegexServiceClass()
        # self.url = 'http://www.66ip.cn/{}.html'
        # self.url = 'https://www.kuaidaili.com/free/inha/{}/'
        self.config = {}
        self.config[ProxySourceType.XICI] ={
            'name': ProxySourceType.XICI,
            'urlList': [
                'https://www.xicidaili.com/nn/{}',
                'https://www.xicidaili.com/nt/{}',
                'https://www.xicidaili.com/wn/{}',
                'https://www.xicidaili.com/wt/{}',
            ],
            'reName': 'tr',
            'reClass': 'odd',
            'reId': None,
            'reSubName': 'td',
        }
        self.config[ProxySourceType.IP66] ={
            'name': ProxySourceType.IP66,
            'urlList': [
                'http://www.66ip.cn/{}.html',
            ],
            'reName': 'tr',
            'reClass': None,
            'reId': None,
            'reSubName': 'td',
        }

    # 抓取某个来源得代理
    def get_all_proxy_by_type(self, proxySourceType, isForce=False):
        proxyList = []
        fileName = './%s-%s.json'%(proxySourceType.name, self.time.GetCurrDayZeroTime(True))
        if self.file.Exists(fileName) and not isForce:
            return json.loads(self.file.ReadFile(fileName))
        config = self.config.get(proxySourceType)
        if not config:
            return False
        self.log.LogD('开始抓取 %s 代理'%config['name'])
        for url in config['urlList']:
            result = self.http.GetUrlResetByRequests(url.format(1), isAutoEncoding = True)
            if result.status_code != 200:
                continue
            soup = BeautifulSoup(result.text, 'html.parser')
            allPageNum = 1
            for item in soup.find_all('a'):
                number = item.text
                if number.isdigit():
                    number = int(number)
                    allPageNum = number if number>allPageNum else allPageNum
            for pageIndex in range(1, allPageNum):
                time.sleep(2)
                result = self.http.GetUrlResetByRequests(url.format(pageIndex), isAutoEncoding = True)
                if result.status_code != 200:
                    continue
                soup = BeautifulSoup(result.text, 'html.parser')
                # print(soup.find_all(config['reName'], id=config['reId'], class_=config['reClass']))
                for item in soup.find_all(config['reName'], id=config['reId'], class_=config['reClass']):
                    # print(item)
                    data = {
                        'type':'',
                        'ip':'',
                        'port':'',
                        'isAnonymous':False,
                        'position':'',
                    }
                    index = 0
                    for itemd in item.find_all(config['reSubName']):
                        text = itemd.text
                        if '高匿' in text or '透明' in text:
                            data['isAnonymous'] = '高匿' in text
                        elif 'http' in text.lower():
                            data['type'] = 'HTTP' if 'http' == text.lower() else 'HTTPS'
                        elif text in self.regex.GetRegexIP(text):
                            data['ip'] = text
                        elif text.isdigit():
                            data['port'] = int(text)
                        else:
                            if index==3 and config['name']==ProxySourceType.XICI:
                                data['position'] = text.replace('\n','')
                        index += 1
                    if not data in proxyList and data.get('ip') != '':
                        proxyList.append(data)
                if pageIndex+1 == allPageNum:
                    percent = 100.0
                    print('%s 代理收集完成 : %s [%d/%d]'%(url, str(percent)+'%',pageIndex+1,allPageNum), end='\n')
                else:
                    percent = round(1.0 * pageIndex / allPageNum * 100,2)
                    print('%s 代理正在收集 : %s [%d/%d]'%(url, str(percent)+'%',pageIndex+1,allPageNum), end='\r')
            '''
            self.log.LogD('开始检查 %s 代理'%config['name'])
            for proxyIndex in range(len(proxyList)-1, -1, -1):
                result = self.http.GetUrlResetByRequests('https://www.baidu.com', timeout=3, proxies={proxyList[proxyIndex]['type']:'%s:%s'%(proxyList[proxyIndex]['ip'],proxyList[proxyIndex]['port'])})
                if not result or result.status_code != 200:
                    proxyList.remove(proxyList[proxyIndex])
            self.log.LogD('结束检查 %s 代理'%config['name'])
            self.log.LogD('%s 可用代理'%proxyList)
            '''

        self.file.WriteFile(fileName, json.dumps(proxyList))
        self.log.LogD('结束抓取 %s 代理'%config['name'])
        return proxyList
        
    # 收集所有得代理
    def collect_all_proxy(self):
            for proxySourceType in ProxySourceType:
                self.get_all_proxy_by_type(proxySourceType)
if __name__ == "__main__":
    ProxyServerClass().collect_all_proxy()